@@ -5,7 +5,7 @@ import os
 
 import sys
 
-sys.path.insert(0, os.path.abspath('../..'))
+sys.path.insert(0, os.path.abspath('../src'))
 
 extensions = [
     'sphinx.ext.autodoc',
@@ -24,7 +24,7 @@ project = 'igem-wikisync'
 year = '2020'
 author = 'Pranav Ballaney'
 copyright = '{0}, {1}'.format(year, author)
-version = release = '0.0.3'
+version = release = '0.0.5'
 
 pygments_style = 'trac'
 templates_path = ['.']
