@@ -5,7 +5,7 @@ from bs4 import BeautifulSoup
 from igem_wikisync.logger import logger
 
 
-def iGEM_login(browser, credentials: dict) -> bool:
+def iGEM_login(browser, credentials: dict, config: dict) -> bool:
     """
     Logs into the iGEM server.
 
@@ -13,6 +13,7 @@ def iGEM_login(browser, credentials: dict) -> bool:
         browser: mechanicalsoup.Browser instance
         credentials: dictionary containing 'username'
             and 'password'
+        config: custom configuration dictionary
 
     Returns:
         True if login is successful.
@@ -20,7 +21,7 @@ def iGEM_login(browser, credentials: dict) -> bool:
     """
 
     # Check if we're already logged in
-    if is_logged_in(browser, credentials['team']):
+    if check_login(browser, config['team'], config['year']):
         logger.info(f"Already logged in as {credentials['username']}.")
         return True
 
@@ -31,8 +32,8 @@ def iGEM_login(browser, credentials: dict) -> bool:
     except Exception:
         message = f"Couldn't connect to {url}."
         logger.debug(message, exc_info=True)
-        logger.error(message)
-        return False
+        logger.critical(message)
+        raise SystemExit
 
     # Check if login was successful
     if response.status_code != 200:
@@ -87,14 +88,14 @@ def iGEM_login(browser, credentials: dict) -> bool:
     return False
 
 
-def is_logged_in(browser, team: str) -> bool:
+def check_login(browser, team: str, year: str) -> bool:
     """
     Check if we're logged into iGEM websites.
     Opens a random iGEM page and checks for edit access.
 
     Arguments:
         browser: mechanicalsoup.Browser instance
-        team: iGEM team name
+        config: custom config dictionary
 
     Returns:
         True if we're logged in.
@@ -102,15 +103,15 @@ def is_logged_in(browser, team: str) -> bool:
     """
 
     # Try opening a random page within the team
-    url = 'https://2020.igem.org/Team:' + team + \
+    url = 'https://' + year + '.igem.org/Team:' + team + \
         '/' + str(datetime.now().microsecond ** 2)
     try:
         browser.open(url)
     except Exception:
         message = "Couldn't connect to iGEM. Please check your internet connection."
         logger.debug(message, exc_info=True)
-        logger.error(message)
-        return False
+        logger.critical(message)
+        raise SystemExit
 
     soup = browser.get_current_page()
 
@@ -172,16 +173,18 @@ def iGEM_upload_page(browser, contents: str, url: str) -> bool:
     return True
 
 
-def iGEM_upload_file(browser, file_object):
+def iGEM_upload_file(browser, file_object, year):
     """
     Upload a file to iGEM servers.
+    iGEM allows files only 100MB large.
+    That check is performed in wikisync.run(), not here.
 
     Parameters:
         browser: mechanicalsoup.Browser instance
         file_object: igem_wikisync.files.OtherFile object
 
     Returns:
-        True is uploaded, False otherwise.
+        True if uploaded, False otherwise.
     """
 
     # Try opening the iGEM upload page
@@ -223,8 +226,8 @@ def iGEM_upload_file(browser, file_object):
     # Extract relative link from response
     relative_link = browser.get_current_page().find(
         class_='fullMedia').find('a')['href']
-    file_object.set_upload_URL('https://2020.igem.org' + relative_link)
+    file_object.set_link_URL('https://' + year + '.igem.org' + relative_link)
 
-    logger.info(f'Uploaded {file_object.upload_filename} to {file_object.upload_URL}.')
+    logger.info(f'Uploaded {file_object.upload_filename} to {file_object.link_URL}.')
 
     return True
