@@ -70,6 +70,9 @@ def iGEM_URL(config: dict, path: Path, upload_map: dict, url: str) -> str:
         Returns false if URL with an unsupported extension is passed
     """
 
+    # Store input for logging and/or returning
+    old_path = url
+
     # Convert to path in case a string was passed
     path = Path(path)
 
@@ -78,10 +81,9 @@ def iGEM_URL(config: dict, path: Path, upload_map: dict, url: str) -> str:
         return url
 
     if url == '/':
-        return 'https://2020.igem.org/Team:' + config['team']
+        return 'https://' + config['year'] + '.igem.org/Team:' + config['team']
 
     # Resolve relative path to local absolute path
-    old_path = url
     resolved_path = resolve_relative_path(url, path.parent, config['src_dir'])
 
     # check upload_map
@@ -97,8 +99,8 @@ def iGEM_URL(config: dict, path: Path, upload_map: dict, url: str) -> str:
         filepath = config['src_dir'] / resolved_path
 
         if not os.path.isfile(filepath):
-            message = f"Warning: {filepath} is referenced in {config['src_dir'] / path} but was not found."
-            logger.error(message)
+            message = f"{filepath} is referenced in {config['src_dir'] / path} but was not found."
+            logger.warning(message)
 
         extension = resolved_path.suffix[1:].lower()
 
@@ -114,10 +116,9 @@ def iGEM_URL(config: dict, path: Path, upload_map: dict, url: str) -> str:
             file_object = JSfile(resolved_path, config)
             url = file_object.link_URL
         else:
-            # TODO: Handle this case
-            logger.error(f"{old_path} is not a source code file and was not found in the upload map.")
-            return False
+            # leave unchanged
+            logger.warning(f"{old_path} is referenced in {path} but was not found.")
+            return old_path
 
     logger.info(f"{old_path} was changed to {url} in {path}.")
-
     return url
