@@ -3,6 +3,7 @@ import shutil
 from hashlib import md5
 from http.cookiejar import LWPCookieJar
 from pathlib import Path
+from datetime import date
 
 import mechanicalsoup
 import yaml
@@ -15,19 +16,25 @@ from igem_wikisync.logger import logger
 # pylint: disable=too-many-instance-attributes, fixme
 
 
-def run(team: str, src_dir: str, build_dir: str):
+def run(team: str,
+        src_dir: str,
+        build_dir: str,
+        year=date.today().year):
     '''
     Runs iGEM-WikiSync and uploads all files to iGEM servers
     while replacing relative URLs with those on the iGEM server.
 
-    Arguments:
-        # @param team: iGEM Team Name
+    Mandatory Arguments:
+        team: iGEM Team Name
+        src_dir: Path to the folder where the source files are present
+        build_dir: Path to the folder where the built files will be stored before uploading
+
+    Optional Arguments:
+        team: iGEM Team Name
         src_dir: Path to the folder where the source files are present
         build_dir: Path to the folder where the built files will be stored before uploading
     '''
 
-    # TODO: does it store files in build_dir also?
-
     # * 1. CHECK AND FORMAT INPUTS
     if team is None or not isinstance(team, str):
         logger.critical('Please specify your team name.')
@@ -46,7 +53,8 @@ def run(team: str, src_dir: str, build_dir: str):
     config = {
         'team':      team,
         'src_dir':   src_dir,
-        'build_dir': build_dir
+        'build_dir': build_dir,
+        'year': str(year)
     }
 
     # * 2. Load or create upload_map
@@ -60,18 +68,17 @@ def run(team: str, src_dir: str, build_dir: str):
     # * 4. Get iGEM credentials from environment variables
     credentials = {
         'username': os.environ.get('IGEM_USERNAME'),
-        'password': os.environ.get('IGEM_PASSWORD'),
-        'team': team
+        'password': os.environ.get('IGEM_PASSWORD')
     }
 
     # * 5. Load/create cookie file
     browser, cookiejar = get_browser_with_cookies()
 
     # * 6. Login to iGEM
-    login = iGEM_login(browser, credentials)
+    login = iGEM_login(browser, credentials, config)
     if not login:
         message = 'Failed to login.'
-        logger.error(message)
+        logger.critical(message)
         raise SystemExit
 
     # * 7. Save cookies
@@ -82,18 +89,20 @@ def run(team: str, src_dir: str, build_dir: str):
     files = cache_files(upload_map, config)
 
     # * 9. Upload all assets and create a map
-    upload_and_write_assets(files['other'], browser, upload_map)
+    uploaded_assets = upload_and_write_assets(files['other'], browser, upload_map, config)
 
     # * 10. write upload map just in case
     # things go wrong while dealing with code
     write_upload_map(upload_map)
 
     # * 11. Build files and upload changed files
-    build_and_upload(files, browser, config, upload_map)
+    uploaded_code = build_and_upload(files, browser, config, upload_map)
 
     # * 12. Write final upload map
     write_upload_map(upload_map)
 
+    print_summary(uploaded_assets, uploaded_code)
+
 
 def get_upload_map():
     """
@@ -108,8 +117,7 @@ def get_upload_map():
             with open('upload_map.yml', 'r') as file:
                 upload_map = yaml.safe_load(file)
         except Exception:
-            logger.critical('upload_map.yml exists but could not be opened.')
-            logger.critical('Please fix/delete the file and run the program again.')
+            logger.critical('upload_map.yml exists but could not be opened. Please try again.')
             raise SystemExit
 
         if isinstance(upload_map, type(None)):
@@ -122,6 +130,7 @@ def get_upload_map():
             elif not isinstance(upload_map[key], dict):
                 logger.critical('upload_map.yml has an invalid format.')
                 logger.critical('Please fix/delete the file and run the program again.')
+                logger.critical('You can find an example of a valid upload map at http://igem-wikisync.rtfd.io.')
                 raise SystemExit
 
         return upload_map
@@ -158,7 +167,7 @@ def get_browser_with_cookies():
         cookiejar: browser cookiejar that can be saved after logging in
     """
 
-    cookie_file = 'igemwiki-upload.cookies'
+    cookie_file = 'wikisync.cookies'
     cookiejar = LWPCookieJar(cookie_file)
     if os.path.exists(cookie_file):
         try:
@@ -221,14 +230,29 @@ def cache_files(upload_map, config):
                                        'xlsx', 'docx', 'pptx', 'csv', 'm', 'ogg', 'gb',
                                        'tif', 'tiff', 'fcs', 'otf', 'eot', 'ttf', 'woff', 'svg']:
 
-                file_object = OtherFile(infile, config)
-                cache['other'][file_object.path] = file_object
+                # make sure file path start with 'assets'
+                if len(str(infile)) < 7 or str(infile)[:7] != 'assets/':
+                    logger.error(f'{infile} is an {extension} file outside the "assets" folder. Skipping.')
+                    continue
+
+                # make sure file size is within limits
+                elif (config['src_dir'] / infile).stat().st_size >= 1000000:
+                    logger.error(f'{infile} is larger than the 100MB file limit. Skipping.')
+                    continue
+                # create OtherFile
+                else:
+                    file_object = OtherFile(infile, config)
+
+                    if len(file_object.upload_filename) < 240:
+                        cache['other'][file_object.path] = file_object
+                    else:
+                        logger.error(f'{infile}: Upload filename too large. Skipping.')
+                        logger.error('Please do not nest assets too deep and take a look at our docs to see how WikiSync renames files.')
+                        continue
 
             else:
-                file_object = None  # just to shut up lintian
-                logger.info(f'{infile} has an unsupported file extension. Skipping.')
-                # ? Do we want to support other text files?
-                # Team lead says no.
+                logger.error(f'{infile} has an unsupported file extension. Skipping.')
+                continue
 
             if extension in ['html', 'css', 'js']:
                 if str(file_object.path) not in upload_map[extension].keys():
@@ -240,7 +264,7 @@ def cache_files(upload_map, config):
     return cache
 
 
-def upload_and_write_assets(other_files, browser, upload_map):
+def upload_and_write_assets(other_files, browser, upload_map, config):
     """"
     Uploads and writes all files and stores URLs in upload_map.
 
@@ -248,66 +272,42 @@ def upload_and_write_assets(other_files, browser, upload_map):
         other_files: dictionary containing OtherFile objects
         browser: mechanicalsoup.StatefulBrowser instance
         upload_map: custom upload map
+        config: custom configuration options
 
     Returns:
-        True if successful
-        Exits if fails
+        Number of files uploaded
+
+    Raises:
+        SystemExit on failure
     """
+
+    # count the number of files uploaded
+    counter = 0
+
     # files have to be uploaded before everything else because
     # the URLs iGEM assigns are random
     for path in other_files.keys():
         file_object = other_files[path]
-        uploaded = False  # flag to keep track of current file upload
+
+        # flag to see if file has already been uploaded
+        uploaded = False
 
         # check if the file has already been uploaded
         for asset_path in upload_map['assets'].keys():
 
+            # if current path matches stored path
             if asset_path == str(path):
                 asset = upload_map['assets'][asset_path]
+                # and the md5 hash is also the same
                 if file_object.md5_hash == asset['md5']:
-                    # ? Can't do anything about renames. iGEM API doesn't allow.
-                    # ? Can find the previous URL and use that itself
-                    # ? but is it worth the effort?
-                    # Team lead says no.
-                    pass
+                    # the file has already been uploaded
+                    uploaded = True
+                    break
                 else:
-                    # If file has changed, write to build_dir
-                    try:
-                        # create directory if doesn't exist
-                        if not os.path.isdir(file_object.build_path.parent):
-                            os.makedirs(file_object.build_path.parent)
-                        shutil.copyfile(file_object.src_path, file_object.build_path)
-                    except Exception:
-                        # print upload map to save the current state
-                        write_upload_map(upload_map)
-                        message = f'Failed to write {str(file_object.path)} to build_dir. ' + \
-                            'The current upload map has been saved. ' + \
-                            'You will not have to upload everything again.'
-                        logger.debug(message, exc_info=True)
-                        logger.error(message)
-                        raise SystemExit
-                        
-
-                    # And upload file and update hash
-                    successful = iGEM_upload_file(browser, file_object)
-                    if not successful:
-                        # print upload map to save the current state
-                        write_upload_map(upload_map)
-                        message = f'Failed to upload {str(file_object.path)}. ' + \
-                            'The current upload map has been saved. ' + \
-                            'You will not have to upload everything again.'
-                        logger.debug(message, exc_info=True)
-                        logger.error(message)
-                        raise SystemExit
-
-                    # TODO: add error handling
-                    asset['md5'] = file_object.md5_hash
-                    asset['link_URL'] = file_object.upload_URL
-
-                    
-
-                uploaded = True
-                break
+                    # the file path matches, but the md5 hash doesn't
+                    # this means the file has changed
+                    uploaded = False
+                    break
 
         # if new file
         if not uploaded:
@@ -316,7 +316,7 @@ def upload_and_write_assets(other_files, browser, upload_map):
                 # create directory if doesn't exist
                 if not os.path.isdir(file_object.build_path.parent):
                     os.makedirs(file_object.build_path.parent)
-                shutil.copyfile(file_object.src_path, file_object.build_path)
+                shutil.copyfile(file_object.src_path, file_object.build_path.parent / file_object.upload_filename)
             except Exception:
                 # print upload map to save the current state
                 write_upload_map(upload_map)
@@ -324,11 +324,10 @@ def upload_and_write_assets(other_files, browser, upload_map):
                     'The current upload map has been saved. ' + \
                     'You will not have to upload everything again.'
                 logger.debug(message, exc_info=True)
-                logger.error(message)
+                logger.critical(message)
                 raise SystemExit
 
-            # and then upload
-            successful = iGEM_upload_file(browser, file_object)
+            successful = iGEM_upload_file(browser, file_object, config['year'])
             if not successful:
                 # print upload map to save the current state
                 write_upload_map(upload_map)
@@ -336,16 +335,22 @@ def upload_and_write_assets(other_files, browser, upload_map):
                 message += 'The current upload map has been saved. '
                 message += 'You will not have to upload everything again.'
                 logger.debug(message, exc_info=True)
-                logger.error(message)
+                logger.critical(message)
                 raise SystemExit
+            else:
+                counter += 1
 
-            upload_map['assets'][str(path)] = {
-                'link_URL': file_object.upload_URL,
-                'md5': file_object.md5_hash,
-                'upload_filename': file_object.upload_filename
-            }
+            if str(path) in upload_map['assets'].keys():
+                upload_map['assets'][str(path)]['md5'] = file_object.md5_hash
+                upload_map['assets'][str(path)]['link_URL'] = file_object.link_URL
+            else:
+                upload_map['assets'][str(path)] = {
+                    'link_URL': file_object.link_URL,
+                    'md5': file_object.md5_hash,
+                    'upload_filename': file_object.upload_filename
+                }
 
-    return True
+    return counter
 
 
 def build_and_upload(files, browser, config, upload_map):
@@ -359,9 +364,15 @@ def build_and_upload(files, browser, config, upload_map):
         upload_map: custom upload map
 
     Returns:
-        Nothing
+        Dictionary with no. of 'html', 'css' and 'js' files uploaded
     """
 
+    counter = {
+        'html': 0,
+        'css': 0,
+        'js': 0,
+    }
+
     for file_dictionary in [files['html'], files['css'], files['js']]:
         for path in file_dictionary.keys():
             file_object = file_dictionary[path]
@@ -406,7 +417,7 @@ def build_and_upload(files, browser, config, upload_map):
                         file.write(processed)
                 except Exception:
                     message = f"Couldn not write {str(file_object.build_path)}. Skipping."
-                    logger.info(message)
+                    logger.error(message)
                     continue
                     # FIXME Can this be improved?
 
@@ -414,6 +425,34 @@ def build_and_upload(files, browser, config, upload_map):
                 successful = iGEM_upload_page(browser, processed, file_object.upload_URL)
                 if not successful:
                     message = f'Could not upload {str(file_object.path)}. Skipping.'
-                    logger.info(message)
+                    logger.error(message)
                     continue
                     # FIXME Can this be improved?
+                else:
+                    counter[ext] += 1
+
+    return counter
+
+
+def print_summary(assets, code):
+
+    total_count = assets + code['html'] + code['css'] + code['js']
+
+    if total_count == 0:
+        print('WikiSync did not find any changes from the previous run. No files were uploaded.')
+    elif total_count == assets:
+        print(f"Done! Successfully uploaded {assets} assets.")
+    elif total_count == code['html']:
+        print(f"Done! Successfully uploaded {code['html']} HTML files.")
+    elif total_count == code['css']:
+        print(f"Done! Successfully uploaded {code['css']} stylesheets.")
+    elif total_count == code['js']:
+        print(f"Done! Successfully uploaded {code['js']} JS scripts.")
+    else:
+        print(f"Done! Successfully uploaded:")
+        print(f"    {assets} assets")
+        print(f"    {code['html']} HTML files")
+        print(f"    {code['css']} stylesheets")
+        print(f"    {code['js']} JS scripts")
+
+    print("Please look at the log for more details.")
\ No newline at end of file
